@@ -6,3 +6,5 @@ set -x
 export PYTHONPATH=./docs/src
 pytest --cov=typer --cov=tests --cov=docs/src --cov-report=term-missing ${@}
 bash ./scripts/lint.sh
+# Check README.md is up to date
+diff --brief docs/index.md README.md
