@@ -1,18 +1,8 @@
-"""Typer, an intuitive CLI library based on optional type hints"""
+"""Typer, build great CLIs. Easy to code. Based on Python type hints."""
 
-__version__ = "0.0.3"
+__version__ = "0.0.6"
 
-from click.exceptions import (  # noqa
-    Abort,
-    BadArgumentUsage,
-    BadOptionUsage,
-    BadParameter,
-    ClickException,
-    FileError,
-    MissingParameter,
-    NoSuchOption,
-    UsageError,
-)
+from click.exceptions import Abort, Exit  # noqa
 from click.termui import (  # noqa
     clear,
     confirm,
@@ -33,7 +23,6 @@ from click.utils import (  # noqa
     format_filename,
     get_app_dir,
     get_binary_stream,
-    get_os_args,
     get_text_stream,
     open_file,
 )
