@@ -25,6 +25,20 @@ from .models import (
     TyperInfo,
 )
 
+try:
+    import click_completion
+    from .completion import _install_completion_placeholder_function
+except ImportError:  # pragma: no cover
+    click_completion = None
+
+
+def get_install_completion_arguments() -> Tuple[click.Parameter, click.Parameter]:
+    signature = inspect.signature(_install_completion_placeholder_function)
+    install_param, show_param = signature.parameters.values()
+    click_install_param, _ = get_click_param(install_param)
+    click_show_param, _ = get_click_param(show_param)
+    return click_install_param, click_show_param
+
 
 class Typer:
     def __init__(
@@ -47,7 +61,9 @@ class Typer:
         add_help_option: bool = Default(True),
         hidden: bool = Default(False),
         deprecated: bool = Default(False),
+        add_completion: bool = True,
     ):
+        self._add_completion = add_completion
         self.info = TyperInfo(
             name=name,
             cls=cls,
@@ -204,14 +220,27 @@ def get_group(typer_instance: Typer) -> click.Command:
 
 
 def get_command(typer_instance: Typer) -> click.Command:
+    if typer_instance._add_completion and click_completion:
+        click_completion.init()
+        click_install_param, click_show_param = get_install_completion_arguments()
     if (
         typer_instance.registered_callback
         or typer_instance.registered_groups
         or len(typer_instance.registered_commands) > 1
     ):
-        return get_group(typer_instance)
+        # Create a Group
+        click_command = get_group(typer_instance)
+        if typer_instance._add_completion and click_completion:
+            click_command.params.append(click_install_param)
+            click_command.params.append(click_show_param)
+        return click_command
     elif len(typer_instance.registered_commands) == 1:
-        return get_command_from_info(typer_instance.registered_commands[0])
+        # Create a single Command
+        click_command = get_command_from_info(typer_instance.registered_commands[0])
+        if typer_instance._add_completion and click_completion:
+            click_command.params.append(click_install_param)
+            click_command.params.append(click_show_param)
+        return click_command
     assert False, "Could not get a command for this Typer instance"
 
 
@@ -230,28 +259,67 @@ def get_group_name(typer_info: TyperInfo) -> Optional[str]:
     return None
 
 
+def solve_typer_info_help(typer_info: TyperInfo) -> str:
+    # Priority 1a: Value was set in app.add_typer()
+    if not isinstance(typer_info.help, DefaultPlaceholder):
+        return inspect.cleandoc(typer_info.help or "")
+    # Priority 1b: Value was set in app.add_typer(), in callback docstring
+    if typer_info.callback:
+        doc = inspect.getdoc(typer_info.callback)
+        if doc:
+            return doc
+    try:
+        # Priority 2a: Value was set in @subapp.callback()
+        doc = typer_info.typer_instance.registered_callback.help
+        if not isinstance(doc, DefaultPlaceholder):
+            return inspect.cleandoc(doc or "")
+        # Priority 2b: Value was set in @subapp.callback(), in callback docstring
+        doc = inspect.getdoc(typer_info.typer_instance.registered_callback.callback)
+        if doc:
+            return doc
+    except AttributeError:
+        pass
+    try:
+        # Priority 3a: Value set in subapp = typer.Typer()
+        instance_value = typer_info.typer_instance.info.help
+        if not isinstance(instance_value, DefaultPlaceholder):
+            return inspect.cleandoc(instance_value or "")
+        doc = inspect.getdoc(typer_info.typer_instance.callback)
+        if doc:
+            return doc
+    except AttributeError:
+        pass
+    # Value not set, use the default
+    return typer_info.help.value
+
+
 def solve_typer_info_defaults(typer_info: TyperInfo) -> TyperInfo:
-    values = {}
+    values: Dict[str, Any] = {}
+    values["help"] = solve_typer_info_help(typer_info)
     name = None
     for name, value in typer_info.__dict__.items():
         # Priority 1: Value was set in app.add_typer()
         if not isinstance(value, DefaultPlaceholder):
             values[name] = value
             continue
-        if typer_info.typer_instance:
-            if typer_info.typer_instance.registered_callback:
-                callback_value = getattr(
-                    typer_info.typer_instance.registered_callback, name
-                )
-                # Priority 2: Value was set in @subapp.callback()
-                if not isinstance(callback_value, DefaultPlaceholder):
-                    values[name] = callback_value
-                    continue
-            instance_value = getattr(typer_info.typer_instance.info, name)
-            # Priority 3: Value set in subapp = typer.Typer()
+        # Priority 2: Value was set in @subapp.callback()
+        try:
+            callback_value = getattr(
+                typer_info.typer_instance.registered_callback, name  # type: ignore
+            )
+            if not isinstance(callback_value, DefaultPlaceholder):
+                values[name] = callback_value
+                continue
+        except AttributeError:
+            pass
+        # Priority 3: Value set in subapp = typer.Typer()
+        try:
+            instance_value = getattr(typer_info.typer_instance.info, name)  # type: ignore
             if not isinstance(instance_value, DefaultPlaceholder):
                 values[name] = instance_value
                 continue
+        except AttributeError:
+            pass
         # Value not set, use the default
         values[name] = value.value
     if values["name"] is None:
@@ -330,6 +398,11 @@ def get_params_convertors_ctx_param_name_from_function(
 def get_command_from_info(command_info: CommandInfo) -> click.Command:
     assert command_info.callback, "A command must have a callback function"
     name = command_info.name or get_command_name(command_info.callback.__name__)
+    use_help = command_info.help
+    if use_help is None:
+        use_help = inspect.getdoc(command_info.callback)
+    else:
+        use_help = inspect.cleandoc(use_help)
     (
         params,
         convertors,
@@ -346,7 +419,7 @@ def get_command_from_info(command_info: CommandInfo) -> click.Command:
             context_param_name=context_param_name,
         ),
         params=params,  # type: ignore
-        help=command_info.help,
+        help=use_help,
         epilog=command_info.epilog,
         short_help=command_info.short_help,
         options_metavar=command_info.options_metavar,
@@ -419,7 +492,7 @@ def get_click_type(
                 min_ = int(parameter_info.min)
             if parameter_info.max is not None:
                 max_ = int(parameter_info.max)
-            return click.IntRange(min=min_, max=max_, clamp=parameter_info.clamp,)
+            return click.IntRange(min=min_, max=max_, clamp=parameter_info.clamp)
         else:
             return click.INT
     elif annotation == float:
